<?php

namespace MyMeals\Services;

use Zeedhi\Framework\DataSource\FilterCriteria;
use MyMeals\Exceptions;
use Teknisa\Libs\Util\InstanceProvider;

class Auth {

    const AUTH_TABLE = "operador";
    const EMAIL_COLUMN = "DSEMAILOPER";

    protected $dataSourceManager;
    protected $entityManager;

    public function __construct($dataSourceManager, $entityManager) {
        $this->dataSourceManager = $dataSourceManager;
        $this->entityManager = $entityManager;
    }

    public function find($filterCriteria) {
        return $this->dataSourceManager->findBy($filterCriteria);
    }

    public function authenticate($email, $password) {
	    $user = $this->getUserByEmail($email);
        
	    if (!$this->isValidUser($user, $password)) {
    		throw new Exceptions\InvalidUserException();
    	}
        return $user;
    }

    public function getUserByEmail($email) {
        $conditions = array();
        $filterCriteria = new FilterCriteria(self::AUTH_TABLE);
        $filterCriteria->addCondition(self::EMAIL_COLUMN, $email);
        $dataSet = $this->find($filterCriteria)->getRows();
    	return isset($dataSet[0]) ? $dataSet[0] : null;
    }

    public function getUserNmFilial($user) {
        $conditions = array(
            array(
                "columnName" => "NRORG",
                "operator" => "=",
                "value" => $user["NRORG"]
            ),
            array(
                "columnName" => "CDFILIAL",
                "operator" => "=",
                "value" => isset($user["CDFILIALSELECIONADA"]) ? $user["CDFILIALSELECIONADA"] : ""
            )
        );
        $filterCriteria = new FilterCriteria("filial", $conditions);
        $dataSet = $this->find($filterCriteria);
        $rows = $dataSet->getRows();
        if (!empty($rows)) {
            return $rows[0];
        } else {
            return array();
        }
    }

    public function isValidUser($user, $password) {
        return $user && $this->isValidPassword($user, $password) && $this->isActiveUser($user);
    }

    private function isActiveUser($user) {
        return $user["IDOPERATIVO"] === "A";
    }

    private function isValidPassword($user, $password) {
    	return $user["CDSENHAOPERWEB"] == md5($password) || $user["CDSENHOPER"] == $this->getCryptoPassword($password);
    }

    private function getCryptoPassword($password) {
        return $this->encrypt($password);
    }
    
    private static function bround($num) {
        if (strpos($num, '.') === false)
            return $num;

        $num = $num;
        $num_arr = explode('.', $num);
        if (empty($num_arr[1])) return $num;
        if($num_arr[1] == 5) {
            if($num_arr[0] % 2) return $num_arr[0]+1;
            else return $num_arr[0];
        } else return round($num);
    }

    public static function encrypt($stString, $inOffset = 2) {
        $inTamanho = intval(strlen($stString));
        $inMeioAnt = intval($inTamanho/2);
        $inMeioPos = intval(self::bround(($inTamanho/2) + 0.1) + 1);

        $stVetor = $stString;

        for($byAux = 1; $byAux <= $inMeioAnt; $byAux++) {
            $chAux = $stVetor[$byAux-1];
            $stVetor[$byAux-1] = $stVetor[$byAux*2-1];
            $stVetor[$byAux*2-1] = $chAux;
        }
        
        for($byAux = $inTamanho; $byAux >= $inMeioPos; $byAux--) {
            $chAux = $stVetor[$byAux*2 - ($inTamanho + 1)-1];
            $stVetor[$byAux*2 - ($inTamanho + 1)-1] = $stVetor[$byAux-1];
            $stVetor[$byAux-1] = $chAux;
        }

        $stString = '';

        for($byAux = 1; $byAux <= $inTamanho; $byAux++) {
            $stString = $stString.(chr(ord($stVetor[$byAux-1]) + $inOffset));
        }

        return $stString;
    }

    public static function decrypt($stString, $inOffset = 2) {
        if($stString == null || $stString == ""){
            return null;
        }
        
        $inTamanho = intval(strlen($stString));
        $inMeioAnt = intval($inTamanho/2); 
        $inMeioPos = intval(self::bround(($inTamanho/2) + 0.1) + 1);

        $stVetor = $stString;

        for($byAux = $inMeioPos; $byAux <= $inTamanho; $byAux++) {
            $chAux = $stVetor[intval($byAux*2 - ($inTamanho + 1))-1];
            $stVetor[intval($byAux*2 - ($inTamanho + 1))-1] = $stVetor[intval($byAux)-1];
            $stVetor[intval($byAux)-1] = $chAux;
        }

        for($byAux = $inMeioAnt; $byAux >= 0; $byAux--) {
            $chAux = $stVetor[$byAux-1];
            $stVetor[$byAux-1] = $stVetor[$byAux*2-1];
            $stVetor[$byAux*2-1] = $chAux;
        }
        
        $stString = '';

        for($byAux = 1; $byAux <= $inTamanho; $byAux++) {
            $stString = $stString.(chr(ord($stVetor[$byAux-1]) - $inOffset));
        }

        return $stString;
    }
    
    public function getNmFilialWithVPD($user) {
        $connection = $this->entityManager->getConnection();
        $newEntities = InstanceProvider::getConnectionCustom()->updateVpdConnection($user['NRORG']);
        if($newEntities) {
            $connection = $newEntities->getConnection();
        }
        $cdfilial = isset($user["CDFILIALSELECIONADA"]) ? $user["CDFILIALSELECIONADA"] : "";
        $SQL = "SELECT NMFILIAL FROM FILIAL WHERE CDFILIAL = :CDFILIAL";
        $params = ['CDFILIAL' => $cdfilial];
        $query = $connection->prepare($SQL);
        $query->execute($params);
        $info = $query->fetchAll();
        if (count($info) > 0) {
            return $info[0]['NMFILIAL'];
        }
        return null;
    }
}