// FILE: js/util/GeneralMethods.js
function GeneralMethods(ScreenService, MessageService) {

    var timeoutErrorCode = "timeout";

    function requestErrorCallback(errorCode, variables, params) {
        if (!params) {
            params = {
                type: "error",
                icon: "error"
            };
        }
        MessageService.getMessageByCode(errorCode).then(function() {
            MessageService.showMessageByCode(errorCode, variables, params);
        }).catch(function() {
            MessageService.showMessageByCode("nao_identificado");
        });
    }

    function catchError(response) {
        let error = response.error;
        if (response.data) {
            error = response.data.error;
        }
        if (!error) {
            throw timeoutErrorCode;
        }

        throw error;
    }

    window.catchError = catchError;
    window.requestErrorCallback = requestErrorCallback;

    var _user = {};

    Object.defineProperty(this, 'user', {
        set: function(obj) {
            _user = obj;
        },
        get: function() {
            return _user;
        }
    });

    this.createAndAppendField = function(widget, fieldProperties) {
        var field = {
            "id": moment().unix(),
            "isVisible": true,
            "showOnForm": true,
            "readOnly": true,
            "readOnlyOnEdit": false,
            "template": "rui-fields#field/text.html",
            "maxlength": 6,
            "class": "12",
            "events": [],
            "validations": {
                "required": true
            }
        };
        field = angular.extend({}, field, fieldProperties);
        metaDataFactory.factoryField(field, widget, true);
    };

    this.createAndAppendFieldNovalidation = function(widget, fieldProperties) {
        var field = {
            "id": moment().unix(),
            "isVisible": true,
            "showOnForm": true,
            "readOnly": true,
            "readOnlyOnEdit": false,
            "template": "rui-fields#field/text.html",
            "maxlength": 6,
            "class": "12",
            "events": []
        };
        field = angular.extend({}, field, fieldProperties);
        metaDataFactory.factoryField(field, widget, true);
    };

    this.createAndAppendFieldButton = function(widget, fieldProperties) {
        var field = {
            "id": moment().unix()
        };
        field = angular.extend({}, field, fieldProperties);
        metaDataFactory.factoryField(field, widget, true);
    };

    this.edit = function(widget) {
        widget.backupRow = angular.copy(widget.currentRow);
        widget.edit();
    };

    this.cancelEdit = function(widget) {
        for (var attr in widget.backupRow) {
            widget.currentRow[attr] = widget.backupRow[attr];
        }
        widget.fields.forEach(function(field) {
            if (field.name != "estadoAprovacao") {
                field.readOnly = true;
            }
        });
        widget.view();
        widget.currentRow.$error = {};
        widget.reload();
        widget.template = "widget/form.html";
    };

    this.valueToFloat = function(value) {
        if (typeof value === 'string' && !Util.isEmptyOrBlank(value)) {
            if (value.indexOf(',') === -1 && value.indexOf('.') === -1) {
                return parseFloat(value);
            } else if (value.indexOf(',') > value.indexOf('.')) {
                return parseFloat(value.replace(/\./g, '').replace(/\,/g, '.'));
            } else if (value.indexOf(',') < value.indexOf('.')) {
                return parseFloat(value.replace(/\,/g, ''));
            }
        } else {
            return value;
        }
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('GeneralMethods', GeneralMethods);
});


// FILE: js/services/AuthService.js
function authService(restEngine) {

    this.validateSession = function() {
        return restEngine.request('/validate_session', {
            "requestType": "Empty"
        }).then(function(response) {
            return response.dataset.user;
        }).catch(catchError);
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register("AuthService", authService);
});


// FILE: js/services/ChangeUnitService.js
function changeUnitService(restEngine) {

    this.saveUserCurrentUnit = function(currentRow) {
        return restEngine.request('/saveUserCurrentUnit', {
            "requestType": "Row",
            "row": currentRow
        }).then(function(response) {
            return response;
        }).catch(catchError);
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register("ChangeUnitService", changeUnitService);
});


// FILE: js/services/LoginService.js
function loginService($http, restEngine, ScreenService, templateManager) {

    var containerNames = ["change-unit", "Informacoes", "meals", "Pratos", "login"];

    this.login = function(row) {
        return restEngine.request('/login', {
            "requestType": "Row",
            "row": row
        }).then(function(response) {
            return response.dataset.user;
        }).catch(catchError);
    };

    this.removeUserData = function() {
        this.removeContainersFromProject(containerNames);
        setLocalVar("Session", "");
    };

    this.testConnection = function(url) {
        return restEngine.request('/testConnection', {
            "requestType": "Empty"
        }, null, null, null, url);
    };

    this.removeContainersFromProject = function(containersToRemove) {
        containersToRemove = containersToRemove || containerNames;
        containersToRemove.forEach(function(container) {
            delete templateManager.containers[container];
        });
    };

    this.compareVersion = function(serverVersion) {
        var status = "OK";
        var appVersion = window.appVersion;
        if (parseFloat(serverVersion) > parseFloat(appVersion)) {
            status = "newVersion";
        } else if (parseFloat(serverVersion) < parseFloat(appVersion)) {
            status = "serverOut";
        }

        return status;
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register("LoginService", loginService);
});


// FILE: js/services/MealsService.js
function mealsService(ZHPromise, restEngine, GeneralMethods) {
    var scope = this;

    this.getDeliveryPointsAndShifts = function(menuDate, serviceCode) {
        var promises = [];
        promises.push(this.getDeliveryPoints(menuDate, serviceCode));
        promises.push(this.getShifts(menuDate, serviceCode));

        return ZHPromise.all(promises)
            .then(function(response) {
                return {
                    deliveryPoints: response[0],
                    shifts: response[1]
                };
            })
            .catch(catchError);
    };

    this.getDeliveryPointsAndShiftsHistory = function(nrhistoricomymeals) {
        return restEngine.post("/getDeliveryPointsAndShiftsHistory", {
            "requestType": "Row",
            "row": nrhistoricomymeals
        });
    };

    this.getDeliveryPoints = function(menuDate, serviceCode) {
        return request("/getDeliveryPoints", "deliveryPoints", menuDate, serviceCode);
    };

    this.getShifts = function(menuDate, serviceCode) {
        return request("/getShifts", "shifts", menuDate, serviceCode);
    };

    this.getPlates = function(menuDate, serviceCode) {
        return request("/getPlates", "plates", menuDate, serviceCode);
    };

    this.save = function(meal, rowEdited, emailMessages) {
        meal.changed = meal.COMENSAIS !== rowEdited.COMENSAIS;
        meal.QTANTERIOR = angular.copy(meal.COMENSAIS);
        meal.COMENSAIS = rowEdited.COMENSAIS;
        updateShifts(rowEdited);
        updateDeliveryPoints(rowEdited);
        return restEngine.request("/meals/save", {
            requestType: "Row",
            row: {
                meal: meal,
                shifts: _shifts,
                deliveryPoints: _deliveryPoints,
                emailMessages: emailMessages
            }
        }).then(function(response) {
            var mealSaved = response.dataset.meal[0];
            for (var attr in mealSaved) {
                meal[attr] = mealSaved[attr];
            }
        }).catch(catchError);
    };

    function updateShifts(rowEdited) {
        for (var attr in rowEdited) {
            if (attr.indexOf("TURNO") !== -1) {
                var CDTURNO = attr.slice(-1);
                for (var i = 0; i < _shifts.length; i++) {
                    if (_shifts[i].CDTURNO === CDTURNO) {
                        _shifts[i].COMENSAIS = rowEdited[attr];
                    }
                }
            }
        }
    }

    function updateDeliveryPoints(rowEdited) {
        for (var attr in rowEdited) {
            if (attr.indexOf("PONTO") !== -1) {
                var CDDESTFILI = attr.slice(-3);
                for (var i = 0; i < _deliveryPoints.length; i++) {
                    if (_deliveryPoints[i].CDDESTFILI === CDDESTFILI) {
                        _deliveryPoints[i].COMENSAIS = rowEdited[attr];
                    }
                }
            }
        }
    }

    this.savePlates = function(changedPlates, meal) {
        updatePlates(changedPlates, meal.COMENSAIS);
        return restEngine.request("/plates/save", {
            requestType: "Row",
            row: {
                meal: meal,
                plates: getChangedPlates()
            }
        }).then(function() {
            for (var i = 0; i < _editablePlates.length; i++) {
                _editablePlates[i].changed = false;
            }
        }).catch(catchError);
    };

    function updatePlates(changedPlates, serviceQuantity) {
        serviceQuantity = GeneralMethods.valueToFloat(serviceQuantity);
        for (var attr in changedPlates) {
            var platesProperties = attr.split("-");
            for (var i = 0; i < _editablePlates.length; i++) {
                if (_editablePlates[i].CDTIPOPRAT === platesProperties[0] && _editablePlates[i].CDPRATO === platesProperties[1]) {
                    var plateQuantity = GeneralMethods.valueToFloat(changedPlates[attr]);
                    _editablePlates[i].QTCOMEORIGINAL = angular.copy(_editablePlates[i].COMENSAIS);
                    _editablePlates[i].COMENSAIS = plateQuantity;
                    _editablePlates[i].VRPECONSCARD = plateQuantity * 100 / serviceQuantity;
                    _editablePlates[i].changed = true;
                    break;
                }
            }
        }
    }

    function getChangedPlates() {
        return _editablePlates.filter(function(plate) {
            return plate.changed;
        });
    }

    function request(route, dataSourceName, menuDate, serviceCode) {
        return restEngine.request(route, {
            requestType: "FilterData",
            filter: getSessionFilter().concat(getMenuDateFilter(menuDate, serviceCode))
        }).then(function(response) {
            return response.dataset[dataSourceName];
        }).catch(catchError);
    }

    function getMenuDateFilter(menuDate, serviceCode) {
        var filter = [
            {
                name: "CDSERVFILI",
                operator: "=",
                value: serviceCode
			}
		];
        filter.push(scope.getMenuDateObjFilter(menuDate));
        return filter;
    }

    /**
     * Retrives a menu date obj filter
     * @param  {String} menuDate
     * @return {Object} filter
     */
    this.getMenuDateObjFilter = function(menuDate) {
        return {
            name: "DTCARDAPIO",
            operator: "=",
            value: menuDate
        };
    };

    function getSessionFilter() {
        return [
            {
                name: "CDFILIAL",
                operator: "=",
                value: "SESSION"
            },
            {
                name: "CDOPERADOR",
                operator: "=",
                value: "SESSION"
            },
            {
                name: "NRORG",
                operator: "=",
                value: "SESSION"
            }
		];
    }

    var _shifts = [];
    var _deliveryPoints = [];
    var _editablePlates = [];

    Object.defineProperty(this, 'shifts', {
        set: function(collection) {
            _shifts = collection;
        },
        get: function() {
            return _shifts;
        }
    });

    Object.defineProperty(this, 'deliveryPoints', {
        set: function(collection) {
            _deliveryPoints = collection;
        },
        get: function() {
            return _deliveryPoints;
        }
    });

    Object.defineProperty(this, 'editablePlates', {
        set: function(collection) {
            _editablePlates = collection;
        },
        get: function() {
            return _editablePlates;
        }
    });

}

Configuration(function(ContextRegister) {
    ContextRegister.register('MealsService', mealsService);
});


// FILE: js/controllers/ChangeUnitController.js
function changeUnitController(ScreenService, GeneralMethods, MessageService, ChangeUnitService, templateManager) {
    var messages = {
        sameUnit: "sameUnit",
        confirmUnitChange: "confirmUnitChange"
    };
    this.onEnter = function(unitWidget) {
        unitWidget.reload();
    };

    this.changeUnit = function(unitWidget) {
        var currentRow = unitWidget.currentRow;
        if (currentRow.CDFILIAL === GeneralMethods.user.cdfilial) {
            showMessageSameUnit(currentRow.CDFILIAL, currentRow.NMFILIAL);
        } else {
            var variables = {
                UNIT: currentRow.CDFILIAL + ' - ' + currentRow.NMFILIAL
            };
            var params = {
                type: "CONFIRMATION",
                icon: "question",
                success: changeUnit.bind(currentRow)
            };
            MessageService.confirmMessageByCode(messages.confirmUnitChange, variables, params);
        }
    };

    function changeUnit() {
        ChangeUnitService.saveUserCurrentUnit(this).then(function(callback) {
            var user = callback.dataset.user;
            GeneralMethods.user.cdfilial = this.CDFILIAL;
            GeneralMethods.user.nmfilial = this.NMFILIAL;
            buildUserAndHeader(user);
            setLocalVar('Session', user.session);
            delete templateManager.containers.meals;
            ScreenService.openWindow("meals");
        }.bind(this)).catch(requestErrorCallback);
    }

    function showMessageSameUnit(unitCode, unitName) {
        var variables = {
            UNIT: unitCode + ' - ' + unitName
        };
        MessageService.showMessageByCode(messages.sameUnit, variables);
    }

}

Configuration(function(ContextRegister) {
    ContextRegister.register('ChangeUnitController', changeUnitController);
});


// FILE: js/controllers/LoginController.js
function loginController(LoginService, ScreenService, MessageService, eventAggregator, AuthService, GeneralMethods, templateManager) {

    var BEFORE_PROJECT_STARTUP = "beforeProjectStartup";
    var INVALID_TOKEN = "invalidToken";
    var LOGOUT = "logout";
    var saasURL = "https://mymeals.teknisa.com/backend/index.php";
    var saasIP = "mymeals.teknisa.com";

    this.login = function(widget) {
        if (getLocalVar("url")) {
            if (widget.isValid()) {
                LoginService.login(widget.currentRow)
                    .then(onLoginSuccess)
                    .catch(requestErrorCallback);
            }
        } else {
            MessageService.showMessageByCode("noServer", "alert");
        }
    };

    this.logout = function() {
        var params = {
            type: "CONFIRMATION",
            icon: "question",
            success: logout
        };
        MessageService.confirmMessageByCode(LOGOUT, null, params);
    };

    function logout() {
        LoginService.removeUserData();
        ScreenService.openWindow("login");
    }

    function onLoginSuccess(user) {
        buildUserAndHeader(user);
        if (user.session) {
            setLocalVar('Session', user.session);
        }
        ScreenService.openWindow("meals");
        GeneralMethods.user = user;

    }

    function buildUserAndHeader(user) {
        var userdesc = {
            NMFILIAL: user.cdfilial + ' - ' + user.nmfilial
        };
        ScreenService.buildUserData(user.nmoperador, [userdesc], userMethod);
    }

    window.buildUserAndHeader = buildUserAndHeader;

    function userMethod() {
        // @todo create user profile window
    }

    this.setUserInformation = function(widget) {
        widget.currentRow = {};
        widget.currentRow.IMAGE = templateManager.userData.image;
        widget.currentRow.NAME = templateManager.userData.name;
        widget.currentRow.NMFILIAL = templateManager.userData.descriptions[0] ? templateManager.userData.descriptions[0].NMFILIAL : "";
    };

    eventAggregator.subscribe(BEFORE_PROJECT_STARTUP, beforeProjectStartup);
    eventAggregator.onRequestError(onRequestError);
    eventAggregator.onRequestStart(onRequestStart);

    function onRequestStart(request) {
        request.requestConfig.headers.Session = getLocalVar("Session");
    }

    function beforeProjectStartup() {
        if (!getLocalVar("url")) {
            setLocalVar("url", saasURL);
            setLocalVar("ip", saasIP);
        }
        templateManager.updateURL(getLocalVar("url"));
        AuthService.validateSession().then(onLoginSuccess).catch(onValidateSessionError);
    }

    function onValidateSessionError() {
        ScreenService.openWindow("login");
        ScreenService.closePopup();
    }

    function onRequestError(response) {
        var error = response.data && response.data.data ? response.data.data : "";
        if (error && error.error && error.error.search(INVALID_TOKEN) !== -1) {
            error.error = INVALID_TOKEN;
            ScreenService.openWindow("login");
        }
    }

    this.configureServerPopup = function(widget) {
        var serverPopup = widget.container.getWidget("configServer");
        serverPopup.fields.forEach(function(field) {
            field.applyDefaultValue();
        });
        ScreenService.openPopup(serverPopup);
    };

    this.configureServer = function(widget) {
        if (widget.isValid()) {
            var row = widget.currentRow;
            var ip = row.ip;
            var path = row.path ? '/' + row.path : '';
            var port = !row.port ? '' : ':' + row.port;
            var url = 'http://' + ip + port + path + window.serviceUrl.replace('..', '');

            templateManager.updateURL(url);

            // do test request
            LoginService.testConnection().then(function(serverStatus) {
                // request success
                finishServerConfig(ip, row, url, serverStatus);
            }, function() {
                var url = 'https://' + ip + port + path + window.serviceUrl.replace('..', '');
                templateManager.updateURL(url);
                LoginService.testConnection().then(function(serverStatus) {
                    finishServerConfig(ip, row, url, serverStatus);
                }, function() {
                    // request fail
                    ScreenService.confirmMessage("Não foi possível conectar ao servidor, deseja manter o IP informado?", 'question', function() {
                        // yes
                        setLocalVar('ip', ip);
                        setLocalVar('port', row.port);
                        setLocalVar('path', row.path);
                        setLocalVar('url', url);
                        // close the ip popup
                        ScreenService.closePopup();
                        templateManager.updateTemplate();

                    }, function() {

                        // no, set last ip if exists
                        if (getLocalVar('url')) {
                            templateManager.updateURL(getLocalVar('url'));
                        }

                        widget.newRow();
                        templateManager.updateTemplate();

                    });
                });
            });
        }
    };

    function finishServerConfig(ip, row, url, serverStatus) {
        setLocalVar('ip', ip);
        setLocalVar('port', row.port);
        setLocalVar('path', row.path);
        setLocalVar('url', url);

        // close the ip popup
        ScreenService.closePopup();
        templateManager.updateTemplate();

        var conexaoValida = LoginService.compareVersion(serverStatus.dataset.conn.version);

        if (conexaoValida === "OK") {
            MessageService.successNotificationByCode("connectionSuccessful");
        } else {
            MessageService.showMessageByCode(conexaoValida);
        }
    }

    this.exitApplication = function() {
        if (RuiNativeInterface) {
            MessageService.confirmMessageByCode("exitAppQuestion").then(function() {
                RuiNativeInterface.closeApp();
            });
        }
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register("LoginController", loginController);
});


// FILE: js/controllers/MealsController.js
function mealsController(ScreenService, MealsService, GeneralMethods, ZHPromise, MessageService, templateManager) {

    var numberMask = {
        type: "number"
    };
    var currencyMask = {
        type: "currency",
        params: {
            symbol: "",
            showSymbol: false,
            thousands: ".",
            decimal: ",",
            precision: 2
        }
    };
    var nonEditablePlate = "X";
    var scope = this;
    /**
     * Open filter if meals widget isn't filtered
     * @param  {Widget} mealsWidget
     * @todo set calendar max date
     */
    this.onEnter = function(mealsWidget) {
        mealsWidget.activate();
        if (!mealsWidget.filtered) {
            var filterWidget = mealsWidget.container.getWidget("filter");
            ScreenService.openPopup(filterWidget);
        }
    };

    /**
     * Validates and apply filter
     * @param  {Widget} filterWidget
     */
    this.applyFilter = function(filterWidget) {
        if (filterWidget.isValid()) {
            var mealsWidget = filterWidget.container.getWidget("meals");
            var dataSourceFilter = getAndUpdateDefaultFilter(mealsWidget);
            dataSourceFilter.push(MealsService.getMenuDateObjFilter(filterWidget.currentRow.DTCARDAPIO));
            mealsWidget.dataSourceFilter = dataSourceFilter;
            mealsWidget.reload().then(function() {
                ScreenService.closePopup(true);
                mealsWidget.filtered = true;
            }, requestErrorCallback);
        }
    };

    this.onEnterHistoricoTotal = function(widgetHistoricoTotal) {
        widgetHistoricoTotal.reload();
    };

    this.carregaHistorico = function(mealsWidget) {
        let currentRowMeals = mealsWidget.currentRow;
        let widgetHistorico = mealsWidget.container.getWidget("historyGrid");
        widgetHistorico.dataSourceFilter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": currentRowMeals.CDFILIAL
            },
            {
                "name": "CDSERVFILI",
                "operator": "=",
                "value": currentRowMeals.CDSERVFILI
            },
            {
                "name": "DTCARDAPIO",
                "operator": "=",
                "value": currentRowMeals.DTCARDAPIO
            },
            {
                "name": "CDOPERADOR",
                "operator": "=",
                "value": currentRowMeals.CDOPERADOR
            }
		];
        widgetHistorico.reload().then(function() {
            scope.selectMeal(mealsWidget.container.getWidget('tabbed'));
        });
    };

    function getAndUpdateDefaultFilter(mealsWidget) {
        var dataSourceFilter = [];
        if (mealsWidget.defaultFilter) {
            dataSourceFilter = angular.copy(mealsWidget.defaultFilter);
        } else {
            dataSourceFilter = [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": GeneralMethods.user.cdfilial
                },
                {
                    "name": "CDOPERADOR",
                    "operator": "=",
                    "value": GeneralMethods.user.cdoperador
                },
                {
                    "name": "NRORG",
                    "operator": "=",
                    "value": GeneralMethods.user.nrorg
                }
            ];
            mealsWidget.defaultFilter = angular.copy(dataSourceFilter);
        }
        return dataSourceFilter;
    }

    this.selectMeal = function(tabbedWidget) {
        tabbedWidget.descriptionOrder = tabbedWidget.parent.descriptionOrder;
        tabbedWidget.container.getWidget("meal").refresh = true;
        tabbedWidget.container.getWidget("plates").refresh = true;
        tabbedWidget.container.getWidget("historyGrid").refresh = true;
        // Swipe aberto
        if (ScreenService.isSwipeOpen()) {
            scope.updateMealWidget(tabbedWidget.container.getWidget("meal"));
            scope.updatePlatesWidget(tabbedWidget.container.getWidget("plates"));
        } else {
            // Abrir swipe
            ScreenService.openSwipe(tabbedWidget);
        }
    };

    this.backToMeals = function(mealsWidget) {
        ScreenService.closeSwipe().then(function() {
            var tabbedWidget = mealsWidget.container.getWidget("tabbed");
            tabbedWidget.lastActiveWidget = mealsWidget;
        });
    };

    this.edit = function(widget) {
        let campoEstadoAprovacao = widget.fields.find(function(campo) {
            return campo.name == "estadoAprovacao";
        });
        if (campoEstadoAprovacao && campoEstadoAprovacao.label == 'PENDENTE') {
            MessageService.showMessageByCode('erroSolicitacaoPendente');
            return;
        } else {
            widget.fields.forEach(function(field) {
                let bloqueio = bloqueioHorarioPreparo(widget);
                if (bloqueio) {
                    MessageService.showMessageByCode('erroBloqTempPrep');
                    return;
                } else {
                    GeneralMethods.edit(widget);
                    if (widget.name === "plates") {
                        var editablePlates = MealsService.editablePlates.map(function(field) {
                            return field.FILTRO + '-' + field.CDPRATO;
                        });
                        if (editablePlates.indexOf(field.name) > -1) {
                            field.readOnly = false;
                        } else {
                            field.readOnly = true;
                        }
                    } else {
                        field.readOnly = false;
                    }
                }
                widget.template = "widget/form.html";
            });
        }
    };

    function bloqueioHorarioPreparo(mealWidget) {
        let currentMeals = mealWidget.parent.parent.currentRow;

        let horasAlteracao = currentMeals.HRALTERACLI;
        if (horasAlteracao) {
            let horas = Number(horasAlteracao.substring(0, 2));
            let minutos = Number(horasAlteracao.substring(2));
            var milisegundosParaSubtrair = (horas * 60 * 60 * 1000) + (minutos * 60 * 1000);

            let dateHoraInicioPreparo = currentMeals.DATA_CMP;
            let parts = dateHoraInicioPreparo.split(/[\/ :]/);

            // Para gerar objeto date o mês é de 0 a 11, por isso subrai 1.
            let month = Number(parts[1]) - 1;

            let dataHoraAtual = new Date();
            let objDateHoraInicioPrep = new Date(parts[2], month, parts[0], parts[3], parts[4], parts[5]);
            let horaLimite = new Date(objDateHoraInicioPrep.getTime() - milisegundosParaSubtrair);

            if (dataHoraAtual >= horaLimite) {
                return true;
            } else {
                return false;
            }
        } else {
            return false;
        }
    }

    this.view = function(widget) {
        widget.fields.forEach(function(field) {
            field.readOnly = true;
        });
        widget.view();
        widget.currentRow.$error = {};
        widget.reload();
        widget.template = "widget/form.html";
    };

    this.updatePlatesWidget = function(platesWidget) {
        if (platesWidget.refresh) {
            var currentMeal = platesWidget.container.getWidget("meals").currentRow;
            var menuDate = moment(currentMeal.DTCARDAPIO, "DD/MM/YYYY H:m:s").format("DD/MM/YYYY");
            platesWidget.fields = [];
            MealsService.getPlates(menuDate, currentMeal.CDSERVFILI).then(function(response) {
                var currentRow = {};
                setPlatesWidgetFields(platesWidget, response, currentRow);
                platesWidget.setCurrentRow(currentRow);
                scope.view(platesWidget);
                platesWidget.refresh = false;
            }, requestErrorCallback);
        }
    };

    function setPlatesWidgetFields(platesWidget, plates, currentRow) {
        var editablePlates = getEditablePlates(plates);
        var nonEditablePlates = getNonEditablePlates(plates);
        MealsService.editablePlates = editablePlates;
        setPlatesFields(platesWidget, editablePlates, currentRow, false);
        setPlatesFields(platesWidget, nonEditablePlates, currentRow, true);
    }

    function getEditablePlates(plates) {
        return plates.filter(function(plate) {
            return plate.FILTRO !== nonEditablePlate;
        });
    }

    function getNonEditablePlates(plates) {
        return plates.filter(function(plate) {
            return plate.FILTRO === nonEditablePlate;
        });
    }

    this.updateMealWidget = function(mealWidget) {
        if (mealWidget.refresh) {
            var currentMeal = mealWidget.container.getWidget("meals").currentRow;
            var menuDate = moment(currentMeal.DTCARDAPIO, "DD/MM/YYYY H:m:s").format("DD/MM/YYYY");
            mealWidget.fields = [];
            scope.view(mealWidget);
            MealsService.getDeliveryPointsAndShifts(menuDate, currentMeal.CDSERVFILI).then(function(response) {
                var currentRow = {};
                setMealWidgetFields(mealWidget, response, currentRow, currentMeal);
                mealWidget.setCurrentRow(currentRow);
                mealWidget.refresh = false;
            }, requestErrorCallback);
        }
    };

    function setMealWidgetFields(mealWidget, obj, currentRow, currentMeal) {
        MealsService.deliveryPoints = obj.deliveryPoints;
        MealsService.shifts = obj.shifts;
        setDefaultField(mealWidget, currentRow, currentMeal);
        setShiftsFields(mealWidget, MealsService.shifts, currentRow);
        setDeliveryPointsFields(mealWidget, MealsService.deliveryPoints, currentRow);
    }

    this.abreObservacao = function(widget) {
        try {
            let shiftfFields = getFieldsByNameLike(widget.fields, "TURNO");
            let deliveryPointsFields = getFieldsByNameLike(widget.fields, "PONTO");
            let rowEdited = widget.currentRow;
            let quantity = parseInt(rowEdited.COMENSAIS);
            let obsWidget = widget.container.getWidget("observacaoWidget");
            validateShifts(shiftfFields, quantity);
            validateDeliveryPoints(deliveryPointsFields, quantity);
            obsWidget.currentRow.OBSERVACAO = " ";
            ScreenService.openPopup(obsWidget);
        } catch (errorCode) {
            MessageService.showMessageByCode(errorCode);
        }
    };

    this.salvarObservacao = function(widget) {
        let observacao = widget.currentRow.OBSERVACAO;
        let mealWidget = widget.container.getWidget("meal");
        scope.saveMeal(mealWidget, observacao);
        ScreenService.closePopup();
    };

    this.defaultFieldsPopUpHistorico = function(widgetHistorico) {
        let popupHistorico = templateManager.container.getWidget("historyDados");
        popupHistorico.currentRow.DTINCLUSAO = widgetHistorico.currentRow.DTINCLUSAO;
        popupHistorico.currentRow.IDSITUACAO = widgetHistorico.currentRow.IDSITUACAO;
        popupHistorico.currentRow.QTCOMENSAIS = widgetHistorico.currentRow.QTCOMENSAIS;
        popupHistorico.currentRow.DSOBSERVACAO = widgetHistorico.currentRow.DSOBSERVACAO;
        popupHistorico.currentRow.DSJUSTIFICATIVA = widgetHistorico.currentRow.DSJUSTIFICATIVA;

        let campoDtinclusao = {
            "name": "DTINCLUSAO",
            "label": "Data",
            "readOnly": true,
            "template": "rui-fields#field/text.html",
            "isVisible": true,
            "dateFormat": "DD/MM/YYYY",
            "displayFormat": "DD/MM/YYYY",
            "defaultValue": "@moment().format('DD/MM/YYYY')@",
            "events": [],
            "class": 6
        };
        GeneralMethods.createAndAppendField(popupHistorico, campoDtinclusao);

        let campoIdsituacao = {
            "name": "IDSITUACAO",
            "label": "Situação",
            "readOnly": true,
            "template": "rui-fields#field/text.html",
            "isVisible": true,
            "showOnForm": true,
            "showOnList": true,
            "showOnView": true,
            "class": 6
        };
        GeneralMethods.createAndAppendField(popupHistorico, campoIdsituacao);

        let campoQtcomensais = {
            "name": "QTCOMENSAIS",
            "label": "Quantidade",
            "readOnly": true,
            "template": "rui-fields#field/text.html",
            "isVisible": true,
            "showOnForm": true,
            "showOnList": true,
            "showOnView": true,
            "class": 12
        };
        GeneralMethods.createAndAppendField(popupHistorico, campoQtcomensais);

        let campoDsobservacao = {
            "name": "DSOBSERVACAO",
            "label": "Observação",
            "readOnly": true,
            "template": "rui-fields#field/text.html",
            "isVisible": true,
            "fieldGroup": "observacao",
            "showOnForm": true,
            "showOnList": true,
            "showOnView": true,
            "class": 12
        };
        GeneralMethods.createAndAppendField(popupHistorico, campoDsobservacao);

        let campoDsjustificativa = {
            "name": "DSJUSTIFICATIVA",
            "label": "Justificativa",
            "readOnly": true,
            "template": "rui-fields#field/text.html",
            "isVisible": true,
            "fieldGroup": "justificativa",
            "showOnForm": true,
            "showOnList": true,
            "showOnView": true,
            "class": 12
        };
        GeneralMethods.createAndAppendField(popupHistorico, campoDsjustificativa);
    };

    this.defaultFieldsPopUpHistoricoTotal = function(widgetHistoricoTotal) {
        let popupHistoricoTotal = templateManager.container.getWidget("historyDadosTotal");
        popupHistoricoTotal.currentRow.DTINCLUSAO = widgetHistoricoTotal.currentRow.DTINCLUSAO;
        popupHistoricoTotal.currentRow.IDSITUACAO = widgetHistoricoTotal.currentRow.IDSITUACAO;
        popupHistoricoTotal.currentRow.QTCOMENSAIS = widgetHistoricoTotal.currentRow.QTCOMENSAIS;
        popupHistoricoTotal.currentRow.DSOBSERVACAO = widgetHistoricoTotal.currentRow.DSOBSERVACAO;
        popupHistoricoTotal.currentRow.DSJUSTIFICATIVA = widgetHistoricoTotal.currentRow.DSJUSTIFICATIVA;

        let campoDtinclusao = {
            "name": "DTINCLUSAO",
            "label": "Data",
            "readOnly": true,
            "template": "rui-fields#field/text.html",
            "isVisible": true,
            "dateFormat": "DD/MM/YYYY",
            "displayFormat": "DD/MM/YYYY",
            "defaultValue": "@moment().format('DD/MM/YYYY')@",
            "events": [],
            "class": 6
        };
        GeneralMethods.createAndAppendField(popupHistoricoTotal, campoDtinclusao);

        let campoIdsituacao = {
            "name": "IDSITUACAO",
            "label": "Situação",
            "readOnly": true,
            "template": "rui-fields#field/text.html",
            "isVisible": true,
            "showOnForm": true,
            "showOnList": true,
            "showOnView": true,
            "class": 6
        };
        GeneralMethods.createAndAppendField(popupHistoricoTotal, campoIdsituacao);

        let campoQtcomensais = {
            "name": "QTCOMENSAIS",
            "label": "Quantidade",
            "readOnly": true,
            "template": "rui-fields#field/text.html",
            "isVisible": true,
            "showOnForm": true,
            "showOnList": true,
            "showOnView": true,
            "class": 12
        };
        GeneralMethods.createAndAppendField(popupHistoricoTotal, campoQtcomensais);

        let campoDsobservacao = {
            "name": "DSOBSERVACAO",
            "label": "Observação",
            "readOnly": true,
            "template": "rui-fields#field/text.html",
            "isVisible": true,
            "fieldGroup": "observacao",
            "showOnForm": true,
            "showOnList": true,
            "showOnView": true,
            "class": 12
        };
        GeneralMethods.createAndAppendField(popupHistoricoTotal, campoDsobservacao);

        let campoDsjustificativa = {
            "name": "DSJUSTIFICATIVA",
            "label": "Justificativa",
            "readOnly": true,
            "template": "rui-fields#field/text.html",
            "isVisible": true,
            "fieldGroup": "justificativa",
            "showOnForm": true,
            "showOnList": true,
            "showOnView": true,
            "class": 12
        };
        GeneralMethods.createAndAppendField(popupHistoricoTotal, campoDsjustificativa);
    };

    this.abreSwipeHistorico = function(widgetHistorico) {
        let popupHistorico = templateManager.container.getWidget("historyDados");
        popupHistorico.fields = [];
        let nrhistoricomymeals = {
            'NRHISTORICOMYMEALS': widgetHistorico.currentRow.NRHISTORICOMYMEALS
        };
        MealsService.getDeliveryPointsAndShiftsHistory(nrhistoricomymeals).then(function(response) {
            scope.defaultFieldsPopUpHistorico(widgetHistorico);
            let turnosRetorno = response.dataset.dadosRetorno.TURNOS;
            let pontosEntregaRetorno = response.dataset.dadosRetorno.PONTO_ENTREGA;
            let comensaisTotalRetorno = response.dataset.dadosRetorno.COMENSAIS[0].QTCOMEALTERADA;
            turnosRetorno.forEach(function(shift) {
                let comensaisTurnos = parseFloat(shift.QTCOMEALTERADA);
                comensaisTurnos = comensaisTurnos >= 0 ? Math.ceil(comensaisTurnos) : 0;
                let propertiesTurnos = {
                    "name": "TURNO" + shift.CDTURNO,
                    "label": shift.NMTURNO,
                    "shifitCode": shift.CDTURNO,
                    "defaultValue": comensaisTurnos,
                    "fieldGroup": "turnos"
                };
                popupHistorico.currentRow[propertiesTurnos.name] = propertiesTurnos.defaultValue;
                GeneralMethods.createAndAppendField(popupHistorico, propertiesTurnos);
            });
            pontosEntregaRetorno.forEach(function(deliveryPoint) {
                var comensais = parseFloat(deliveryPoint.QTCOMEALTERADA);
                comensais = comensais >= 0 ? Math.ceil(comensais) : 0;
                var properties = {
                    "name": "PONTO" + deliveryPoint.CDDESTFILI,
                    "label": deliveryPoint.NMDESTFILI,
                    "deliveryPointCode": deliveryPoint.CDDESTFILI,
                    "defaultValue": comensais,
                    "fieldGroup": "pontosEntrega"
                };
                popupHistorico.currentRow[properties.name] = properties.defaultValue;
                GeneralMethods.createAndAppendField(popupHistorico, properties);
                popupHistorico.currentRow.QTCOMENSAIS = comensaisTotalRetorno;
            });
        }, requestErrorCallback);
        ScreenService.openPopup(popupHistorico);
    };

    this.abreSwipeHistoricoTotal = function(widgetHistoricoTotal) {
        var popupHistoricoTotal = templateManager.container.getWidget("historyDadosTotal");
        popupHistoricoTotal.fields = [];
        let nrhistoricomymeals = {
            'NRHISTORICOMYMEALS': widgetHistoricoTotal.currentRow.NRHISTORICOMYMEALS
        };
        MealsService.getDeliveryPointsAndShiftsHistory(nrhistoricomymeals).then(function(response) {
            scope.defaultFieldsPopUpHistoricoTotal(widgetHistoricoTotal);
            let turnosRetorno = response.dataset.dadosRetorno.TURNOS;
            let pontosEntregaRetorno = response.dataset.dadosRetorno.PONTO_ENTREGA;
            let comensaisTotalRetorno = response.dataset.dadosRetorno.COMENSAIS[0].QTCOMEALTERADA;
            turnosRetorno.forEach(function(shift) {
                let comensaisTurnos = parseFloat(shift.QTCOMEALTERADA);
                comensaisTurnos = comensaisTurnos >= 0 ? Math.ceil(comensaisTurnos) : 0;
                let propertiesTurnos = {
                    "name": "TURNO" + shift.CDTURNO,
                    "label": shift.NMTURNO,
                    "shifitCode": shift.CDTURNO,
                    "defaultValue": comensaisTurnos,
                    "fieldGroup": "turnos"
                };
                popupHistoricoTotal.currentRow[propertiesTurnos.name] = propertiesTurnos.defaultValue;
                GeneralMethods.createAndAppendField(popupHistoricoTotal, propertiesTurnos);
            });
            pontosEntregaRetorno.forEach(function(deliveryPoint) {
                var comensais = parseFloat(deliveryPoint.QTCOMEALTERADA);
                comensais = comensais >= 0 ? Math.ceil(comensais) : 0;
                var properties = {
                    "name": "PONTO" + deliveryPoint.CDDESTFILI,
                    "label": deliveryPoint.NMDESTFILI,
                    "deliveryPointCode": deliveryPoint.CDDESTFILI,
                    "defaultValue": comensais,
                    "fieldGroup": "pontosEntrega"
                };
                popupHistoricoTotal.currentRow[properties.name] = properties.defaultValue;
                GeneralMethods.createAndAppendField(popupHistoricoTotal, properties);
                popupHistoricoTotal.currentRow.QTCOMENSAIS = comensaisTotalRetorno;
            });
        }, requestErrorCallback);
        ScreenService.openPopup(popupHistoricoTotal);
    };

    this.mostraJustificativaReprovado = function() {
        let widgetHistorico = templateManager.container.getWidget("historyGrid");
        let dadosHistorico = widgetHistorico.dataSource.data;
        let labelEstadoAprovacao = dadosHistorico[0].IDSITUACAO;
        let textoJustificativa = dadosHistorico[0].DSJUSTIFICATIVA;

        if (labelEstadoAprovacao == 'REPROVADO') {
            ScreenService.showMessage(textoJustificativa);
        }
    };

    function setDefaultField(mealWidget, currentRow, currentMeal) {
        var comensais = parseFloat(currentMeal.COMENSAIS);
        var widgetHistorico = mealWidget.container.getWidget("historyGrid");
        var dadosHistorico = widgetHistorico.dataSource.data;

        if (dadosHistorico.length > 0) {
            var labelEstadoAprovacao = dadosHistorico[0].IDSITUACAO;
            var botaoStatusAprovacao = {
                "name": "estadoAprovacao",
                "label": labelEstadoAprovacao,
                "class": "2 estadoAprovacao-btn",
                "isVisible": true,
                "template": "rui-fields#field/button.html",
                "events": [
                    {
                        "name": "ActionEvent",
                        "code": "MealsController.mostraJustificativaReprovado();"
                    }
                ]
            };
            if (labelEstadoAprovacao == 'PENDENTE') {
                mealWidget.actions.forEach(function(obj) {
                    if (obj.name === "edit") {
                        obj.readOnly = true;
                    }
                });
            }

            if (labelEstadoAprovacao == 'REPROVADO') {
                GeneralMethods.createAndAppendFieldButton(mealWidget, botaoStatusAprovacao);
            } else {
                GeneralMethods.createAndAppendFieldNovalidation(mealWidget, botaoStatusAprovacao);
            }

        }

        comensais = comensais >= 0 ? Math.ceil(comensais) : 0;
        var properties = {
            "name": "COMENSAIS",
            "label": "Quantity",
            "defaultValue": comensais,
            "mask": numberMask,
            "type": "tel",
            "validations": {
                "required": true
            }
        };
        currentRow[properties.name] = properties.defaultValue;
        GeneralMethods.createAndAppendField(mealWidget, properties);
    }

    function setShiftsFields(mealWidget, shifts, currentRow) {
        shifts.forEach(function(shift) {
            var comensais = parseFloat(shift.COMENSAIS);
            comensais = comensais >= 0 ? Math.ceil(comensais) : 0;
            var properties = {
                "name": "TURNO" + shift.CDTURNO,
                "label": shift.NMTURNO,
                "shifitCode": shift.CDTURNO,
                "defaultValue": comensais,
                "fieldGroup": "shifts",
                "type": "tel",
                "mask": numberMask
            };
            currentRow[properties.name] = properties.defaultValue;
            GeneralMethods.createAndAppendField(mealWidget, properties);
        });
        var isVisible = shifts.length ? true : false;
        toggleFieldGroupVisibility(mealWidget, "shifts", isVisible);
    }

    function setDeliveryPointsFields(mealWidget, deliveryPoints, currentRow) {
        deliveryPoints.forEach(function(deliveryPoint) {
            var comensais = parseFloat(deliveryPoint.COMENSAIS);
            comensais = comensais >= 0 ? Math.ceil(comensais) : 0;
            var properties = {
                "name": "PONTO" + deliveryPoint.CDDESTFILI,
                "label": deliveryPoint.NMDESTFILI,
                "deliveryPointCode": deliveryPoint.CDDESTFILI,
                "defaultValue": comensais,
                "fieldGroup": "deliveryPoints",
                "type": "tel",
                "mask": numberMask
            };
            currentRow[properties.name] = properties.defaultValue;
            GeneralMethods.createAndAppendField(mealWidget, properties);
        });
        var isVisible = deliveryPoints.length ? true : false;
        toggleFieldGroupVisibility(mealWidget, "deliveryPoints", isVisible);
    }

    function setPlatesFields(platesWidget, plates, currentRow) {
        plates.forEach(function(plate) {
            var properties = {
                "name": plate.CDTIPOPRAT + "-" + plate.CDPRATO,
                "label": plate.NMPRATO,
                "defaultValue": plate.COMENSAIS,
                "readOnly": true,
                "mask": currencyMask
            };
            currentRow[properties.name] = properties.defaultValue;
            GeneralMethods.createAndAppendField(platesWidget, properties);
        });
    }

    function toggleFieldGroupVisibility(widget, name, isVisible) {
        for (var index in widget.fieldGroups) {
            var fieldGroup = widget.fieldGroups[index];
            if (fieldGroup.name === name) {
                fieldGroup.isVisible = isVisible;
            }
        }
    }

    this.saveMeal = function(mealWidget, observacao) {
        try {
            if (mealWidget.isValid()) {
                var rowEdited = mealWidget.currentRow;
                var quantity = parseInt(rowEdited.COMENSAIS);
                var currentMeal = mealWidget.parent.parent.currentRow;
                currentMeal.OBSERVACAO = observacao;
                var emailMessages;
                emailMessages = ['emailSubject', 'emailFromName', 'emailAcima',
                'emailAbaixo', 'emailDescription', 'emailFooter',
                'emailPendente', 'emailAprovada', 'emailDataCardapio',
                'emailOperador', 'emailUnidade', 'emailServico',
                'emailQtOri', 'emailQtAlt', 'emailSituacao'];
                var emailMessagesObj;
                emailMessagesObj = {};
                var getMessage;
                getMessage = function(idx) {
                    return MessageService.getMessageByCode(idx).then(function(message) {
                        emailMessagesObj[idx] = message;
                    });
                };
                var emailMessagesPromises;
                emailMessagesPromises = emailMessages.map(getMessage);
                if (!quantity || quantity === 0) {
                    MessageService.confirmMessageByCode('desejaZerarComensais').then(
                        function() {
                            window.Promise.all(emailMessagesPromises).then(function() {
                                MealsService.save(currentMeal, rowEdited, emailMessagesObj).then(function() {
                                    scope.view(mealWidget);
                                    mealWidget.container.getWidget("plates").refresh = true;
                                    mealWidget.container.getWidget("historyGrid").reload().then(function() {
                                        mealWidget.fields = [];
                                        mealWidget.refresh = true;
                                        scope.updateMealWidget(mealWidget);
                                    });
                                }).catch(requestErrorCallback);
                            });
                        },
                        function() {
                            return;
                        }
                    );
                } else {
                    window.Promise.all(emailMessagesPromises).then(function() {
                        MealsService.save(currentMeal, rowEdited, emailMessagesObj).then(function() {
                            scope.view(mealWidget);
                            mealWidget.container.getWidget("plates").refresh = true;
                            mealWidget.container.getWidget("historyGrid").reload().then(function() {
                                mealWidget.fields = [];
                                mealWidget.refresh = true;
                                scope.updateMealWidget(mealWidget);
                            });
                        }).catch(requestErrorCallback);
                    });
                }
            }
        } catch (errorCode) {
            MessageService.showMessageByCode(errorCode);
        }
    };

    function getFieldsByNameLike(fields, name) {
        return fields.filter(function(field) {
            return field.name.indexOf(name) !== -1;
        });
    }

    function validateShifts(shiftfFields, quantity) {
        if (shiftfFields.length) {
            var shiftsSum = getSum(shiftfFields);
            if (shiftsSum !== quantity) {
                throw "shiftNotEqualQuantity";
            }
        }
    }

    function validateDeliveryPoints(deliveryPointsFields, quantity) {
        if (deliveryPointsFields.length) {
            var deliveryPointsSum = getSum(deliveryPointsFields);
            if (deliveryPointsSum !== quantity) {
                throw "deliveryPointsNotEqualQuantity";
            }
        }
    }

    function getSum(fields) {
        return fields.map(function(field) {
            return parseInt(field.value());
        }).reduce(function(lastValue, currentValue) {
            return lastValue + currentValue;
        }, 0);
    }

    this.savePlates = function(platesWidget) {
        try {
            if (platesWidget.isValid()) {
                var changedPlates = getChangedPlates(platesWidget.currentRow, platesWidget.backupRow);
                if (Object.keys(changedPlates).length) {
                    var currentMeal = platesWidget.parent.parent.currentRow;
                    MealsService.savePlates(changedPlates, currentMeal).then(function() {
                        platesWidget.refresh = true;
                        scope.updatePlatesWidget(platesWidget);
                    }).catch(requestErrorCallback);
                } else {
                    GeneralMethods.cancelEdit(platesWidget);
                }
            }
        } catch (errorCode) {
            MessageService.showMessageByCode(errorCode);
        }
    };

    function getChangedPlates(currentRow, backupRow) {
        var changedPlates = {};
        for (var attr in backupRow) {
            if (attr !== "$error" && currentRow[attr] !== backupRow[attr]) {
                changedPlates[attr] = currentRow[attr];
            }
        }
        return changedPlates;
    }

    this.onCloseSwipe = function(tabbedWidget) {
        var mealWidget = tabbedWidget.container.getWidget("meal");
        var platesWidget = tabbedWidget.container.getWidget("plates");
        var mealsWidget = tabbedWidget.container.getWidget("meals");
        GeneralMethods.cancelEdit(mealWidget);
        GeneralMethods.cancelEdit(platesWidget);
        tabbedWidget.lastActiveWidget = mealsWidget;
    };

    this.mealsOnBack = function(widget) {
        if (widget.mode === "edit") {
            GeneralMethods.cancelEdit(widget);
        } else {
            this.backToMeals(widget);
        }
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('MealsController', mealsController);
});

